@@ -3,8 +3,8 @@
 RSpec.describe JWT do
   let(:payload) { { 'user_id' => 'some@user.tld' } }
 
-  let :data do
-    data = {
+  let(:data) do
+    {
       :empty_token => 'e30K.e30K.e30K',
       :empty_token_2_segment => 'e30K.e30K.',
       :invalid_header_token => 'W10.e30K.e30K',
@@ -25,7 +25,6 @@ RSpec.describe JWT do
       'ES256K_public' => test_pkey('ec256k-public.pem'),
       'NONE' => 'eyJhbGciOiJub25lIn0.eyJ1c2VyX2lkIjoic29tZUB1c2VyLnRsZCJ9.',
       'HS256' => 'eyJhbGciOiJIUzI1NiJ9.eyJ1c2VyX2lkIjoic29tZUB1c2VyLnRsZCJ9.kWOVtIOpWcG7JnyJG0qOkTDbOy636XrrQhMm_8JrRQ8',
-      'HS512256' => 'eyJhbGciOiJIUzUxMjI1NiJ9.eyJ1c2VyX2lkIjoic29tZUB1c2VyLnRsZCJ9.Ds_4ibvf7z4QOBoKntEjDfthy3WJ-3rKMspTEcHE2bA',
       'HS384' => 'eyJhbGciOiJIUzM4NCJ9.eyJ1c2VyX2lkIjoic29tZUB1c2VyLnRsZCJ9.VuV4j4A1HKhWxCNzEcwc9qVF3frrEu-BRLzvYPkbWO0LENRGy5dOiBQ34remM3XH',
       'HS512' => 'eyJhbGciOiJIUzUxMiJ9.eyJ1c2VyX2lkIjoic29tZUB1c2VyLnRsZCJ9.8zNtCBTJIZTHpZ-BkhR-6sZY1K85Nm5YCKqV3AxRdsBJDt_RR-REH2db4T3Y0uQwNknhrCnZGvhNHrvhDwV1kA',
       'RS256' => 'eyJhbGciOiJSUzI1NiJ9.eyJ1c2VyX2lkIjoic29tZUB1c2VyLnRsZCJ9.eSXvWP4GViiwUALj_-qTxU68I1oM0XjgDsCZBBUri2Ghh9d75QkVDoZ_v872GaqunN5A5xcnBK0-cOq-CR6OwibgJWfOt69GNzw5RrOfQ2mz3QI3NYEq080nF69h8BeqkiaXhI24Q51joEgfa9aj5Y-oitLAmtDPYTm7vTcdGufd6AwD3_3jajKBwkh0LPSeMtbe_5EyS94nFoEF9OQuhJYjUmp7agsBVa8FFEjVw5jEgVqkvERSj5hSY4nEiCAomdVxIKBfykyi0d12cgjhI7mBFwWkPku8XIPGZ7N8vpiSLdM68BnUqIK5qR7NAhtvT7iyLFgOqhZNUQ6Ret5VpQ',
@@ -38,18 +37,6 @@ RSpec.describe JWT do
       'PS384' => '',
       'PS512' => ''
     }
-
-    if JWT.rbnacl?
-      ed25519_private = RbNaCl::Signatures::Ed25519::SigningKey.new('abcdefghijklmnopqrstuvwxyzABCDEF')
-      ed25519_public =  ed25519_private.verify_key
-      data.merge!(
-        'ED25519_private' => ed25519_private,
-        'ED25519_public' => ed25519_public,
-        'EdDSA_private' => ed25519_private,
-        'EdDSA_public' => ed25519_public
-      )
-    end
-    data
   end
 
   after(:each) do
@@ -106,10 +93,7 @@ RSpec.describe JWT do
     end
   end
 
-  algorithms = %w[HS256 HS384 HS512]
-  algorithms << 'HS512256' if JWT.rbnacl?
-
-  algorithms.each do |alg|
+  %w[HS256 HS384 HS512].each do |alg|
     context "alg: #{alg}" do
       it 'should generate a valid token' do
         token = JWT.encode payload, data[:secret], alg
@@ -178,45 +162,11 @@ RSpec.describe JWT do
     end
   end
 
-  if defined?(RbNaCl)
-    %w[ED25519 EdDSA].each do |alg|
-      context "alg: #{alg}" do
-        before(:each) do
-          data[alg] = JWT.encode payload, data["#{alg}_private"], alg
-        end
-
-        let(:wrong_key) { RbNaCl::Signatures::Ed25519::SigningKey.generate.verify_key }
-
-        it 'should generate a valid token' do
-          jwt_payload, header = JWT.decode data[alg], data["#{alg}_public"], true, algorithm: alg
-
-          expect(header['alg']).to eq alg
-          expect(jwt_payload).to eq payload
-        end
-
-        it 'should decode a valid token' do
-          jwt_payload, header = JWT.decode data[alg], data["#{alg}_public"], true, algorithm: alg
-
-          expect(header['alg']).to eq alg
-          expect(jwt_payload).to eq payload
-        end
-
-        it 'wrong key should raise JWT::DecodeError' do
-          expect do
-            JWT.decode data[alg], wrong_key, true, algorithm: alg
-          end.to raise_error JWT::DecodeError
-        end
-
-        it 'wrong key and verify = false should not raise JWT::DecodeError' do
-          expect do
-            JWT.decode data[alg], wrong_key, false
-          end.not_to raise_error
-        end
-      end
-    end
-  end
-
   %w[ES256 ES384 ES512 ES256K].each do |alg|
+    before do
+      skip 'OpenSSL gem missing RSA-PSS support' unless OpenSSL::PKey::RSA.method_defined?(:sign_pss)
+    end
+
     context "alg: #{alg}" do
       before(:each) do
         data[alg] = JWT.encode(payload, data["#{alg}_private"], alg)
@@ -252,64 +202,51 @@ RSpec.describe JWT do
     end
   end
 
-  if Gem::Version.new(OpenSSL::VERSION) >= Gem::Version.new('2.1')
-    %w[PS256 PS384 PS512].each do |alg|
-      context "alg: #{alg}" do
-        before(:each) do
-          data[alg] = JWT.encode payload, data[:rsa_private], alg
-        end
-
-        let(:wrong_key) { data[:wrong_rsa_public] }
-
-        it 'should generate a valid token' do
-          token = data[alg]
-
-          header, body, signature = token.split('.')
-
-          expect(header).to eql(Base64.strict_encode64({ alg: alg }.to_json))
-          expect(body).to   eql(Base64.strict_encode64(payload.to_json))
-
-          # Validate signature is made of up header and body of JWT
-          translated_alg  = alg.gsub('PS', 'sha')
-          valid_signature = data[:rsa_public].verify_pss(
-            translated_alg,
-            JWT::Base64.url_decode(signature),
-            [header, body].join('.'),
-            salt_length: :auto,
-            mgf1_hash: translated_alg
-          )
-          expect(valid_signature).to be true
-        end
-
-        it 'should decode a valid token' do
-          jwt_payload, header = JWT.decode data[alg], data[:rsa_public], true, algorithm: alg
-
-          expect(header['alg']).to eq alg
-          expect(jwt_payload).to eq payload
-        end
-
-        it 'wrong key should raise JWT::DecodeError' do
-          expect do
-            JWT.decode data[alg], wrong_key
-          end.to raise_error JWT::DecodeError
-        end
-
-        it 'wrong key and verify = false should not raise JWT::DecodeError' do
-          expect do
-            JWT.decode data[alg], wrong_key, false
-          end.not_to raise_error
-        end
+  %w[PS256 PS384 PS512].each do |alg|
+    context "alg: #{alg}" do
+      before(:each) do
+        data[alg] = JWT.encode payload, data[:rsa_private], alg
       end
-    end
-  else
-    %w[PS256 PS384 PS512].each do |alg|
-      context "alg: #{alg}" do
-        it 'raises error about OpenSSL version' do
-          expect { JWT.encode payload, data[:rsa_private], alg }.to raise_error(
-            JWT::RequiredDependencyError,
-            /You currently have OpenSSL .*. PS support requires >= 2.1/
-          )
-        end
+
+      let(:wrong_key) { data[:wrong_rsa_public] }
+
+      it 'should generate a valid token' do
+        token = data[alg]
+
+        header, body, signature = token.split('.')
+
+        expect(header).to eql(Base64.strict_encode64({ alg: alg }.to_json))
+        expect(body).to   eql(Base64.strict_encode64(payload.to_json))
+
+        # Validate signature is made of up header and body of JWT
+        translated_alg  = alg.gsub('PS', 'sha')
+        valid_signature = data[:rsa_public].verify_pss(
+          translated_alg,
+          JWT::Base64.url_decode(signature),
+          [header, body].join('.'),
+          salt_length: :auto,
+          mgf1_hash: translated_alg
+        )
+        expect(valid_signature).to be true
+      end
+
+      it 'should decode a valid token' do
+        jwt_payload, header = JWT.decode data[alg], data[:rsa_public], true, algorithm: alg
+
+        expect(header['alg']).to eq alg
+        expect(jwt_payload).to eq payload
+      end
+
+      it 'wrong key should raise JWT::DecodeError' do
+        expect do
+          JWT.decode data[alg], wrong_key
+        end.to raise_error JWT::DecodeError
+      end
+
+      it 'wrong key and verify = false should not raise JWT::DecodeError' do
+        expect do
+          JWT.decode data[alg], wrong_key, false
+        end.not_to raise_error
       end
     end
   end
@@ -483,6 +420,16 @@ RSpec.describe JWT do
         end.to raise_error JWT::IncorrectAlgorithm
       end
 
+      it 'raises error when keyfinder does not find anything' do
+        token = JWT.encode(payload, 'secret', 'HS256')
+
+        expect do
+          JWT.decode(token, nil, true, algorithm: 'HS256') do
+            nil
+          end
+        end.to raise_error JWT::DecodeError, 'No verification key available'
+      end
+
       it 'should raise JWT::IncorrectAlgorithm when algorithms array does not contain algorithm' do
         token = JWT.encode payload, data[:secret], 'HS512'
 
@@ -606,14 +553,14 @@ RSpec.describe JWT do
   end
 
   context 'a token with invalid Base64 segments' do
-    it 'raises JWT::DecodeError' do
-      expect { JWT.decode('hello.there.world') }.to raise_error(JWT::DecodeError, 'Invalid segment encoding')
+    it 'raises JWT::Base64DecodeError' do
+      expect { JWT.decode('hello.there.world') }.to raise_error(JWT::Base64DecodeError, 'Invalid base64 encoding')
     end
   end
 
   context 'a token with two segments but does not require verifying' do
     it 'raises something else than "Not enough or too many segments"' do
-      expect { JWT.decode('ThisIsNotAValidJWTToken.second', nil, false) }.to raise_error(JWT::DecodeError, 'Invalid segment encoding')
+      expect { JWT.decode('ThisIsNotAValidJWTToken.second', nil, false) }.to raise_error(JWT::Base64DecodeError, 'Invalid base64 encoding')
     end
   end
 
@@ -640,9 +587,9 @@ RSpec.describe JWT do
   end
 
   context 'when the alg value is given as a header parameter' do
-    it 'does not override the actual algorithm used' do
+    it 'overrides the actual algorithm used' do
       headers = JSON.parse(JWT::Base64.url_decode(JWT.encode('Hello World', 'secret', 'HS256', { alg: 'HS123' }).split('.').first))
-      expect(headers['alg']).to eq('HS256')
+      expect(headers['alg']).to eq('HS123')
     end
 
     it 'should generate the same token' do
@@ -790,8 +737,8 @@ RSpec.describe JWT do
 
   context 'when token ends with a newline char' do
     let(:token) { "#{JWT.encode(payload, 'secret', 'HS256')}\n" }
-    it 'ignores the newline and decodes the token' do
-      expect(JWT.decode(token, 'secret', true, algorithm: 'HS256')).to include(payload)
+    it 'raises an error' do
+      expect { JWT.decode(token, 'secret', true, algorithm: 'HS256') }.to raise_error(JWT::Base64DecodeError, 'Invalid base64 encoding')
     end
   end
 
@@ -810,8 +757,9 @@ RSpec.describe JWT do
     let(:token) { JWT.encode(payload, 'secret', 'HS256') }
 
     it 'starts trying with the algorithm referred in the header' do
-      expect(JWT::JWA::Rsa).not_to receive(:verify)
+      allow(JWT::JWA::Rsa).to receive(:verify)
       JWT.decode(token, 'secret', true, algorithm: %w[RS512 HS256])
+      expect(JWT::JWA::Rsa).not_to have_received(:verify)
     end
   end
 
@@ -953,9 +901,8 @@ RSpec.describe JWT do
         end
       end
 
-      it 'emits a deprecation warning' do
-        expect { token }.to output(/.*Custom algorithms are required to include JWT::JWA::SigningAlgorithm.*/).to_stderr
-        expect(JWT.decode(token, 'secret', true, algorithm: custom_algorithm.new)).to eq([payload, { 'alg' => 'custom', 'foo' => 'bar' }])
+      it 'raises an error' do
+        expect { token }.to raise_error(ArgumentError, 'Custom algorithms are required to include JWT::JWA::SigningAlgorithm')
       end
     end
 
@@ -1003,34 +950,4 @@ RSpec.describe JWT do
       end
     end
   end
-
-  context 'when invalid token is valid loose base64' do
-    it 'does not output deprecations warnings' do
-      expect do
-        JWT.decode("#{JWT.encode('a', 'b')} 9", 'b')
-      rescue JWT::VerificationError
-        nil
-      end.not_to output(/DEPRECATION/).to_stderr
-    end
-  end
-
-  context 'when valid token is invalid strict base64 and decoded with the correct key' do
-    it 'does outputs deprecation warning' do
-      expect { JWT.decode("#{JWT.encode('payload', 'key')} ", 'key') }.to output(/DEPRECATION/).to_stderr
-    end
-  end
-
-  context 'when valid token is invalid strict base64 and decoded with the incorrect key' do
-    it 'does not output deprecation warning, even when decoded with the correct key' do
-      token = JWT.encode('payload', 'key')
-      expect do
-        begin
-          JWT.decode("#{token} ", 'incorrect')
-        rescue JWT::VerificationError
-          nil
-        end
-        JWT.decode(token, 'key')
-      end.not_to output(/DEPRECATION/).to_stderr
-    end
-  end
 end
